@@ -15,8 +15,24 @@ Puppet::Type.newtype(:vs_config) do
     end
   end
 
+  newparam(:skip_if_version) do
+    desc 'Skip setting the value when ovs version matches'
+    validate do |value|
+      unless value.is_a?(String) and value =~ /^\d+.\d+$/
+        raise ArgumentError, "Invalid format for  #{value}. Requires a String with format \d+.\d+, not a #{value.class}"
+      end
+    end
+  end
+
+  newparam(:wait) do
+    desc 'Should it wait for ovs-vswitchd to reconfigure itself before it exits'
+
+    newvalues(:true, :false)
+    defaultto :true
+  end
+
   newproperty(:value) do
-    desc 'Configuration value for the paramter'
+    desc 'Configuration value for the parameter'
 
     validate do |value|
       if !value.is_a?(String)
