@@ -4,33 +4,8 @@ describe 'vswitch::ovs' do
 
   let :default_params do {
     :package_ensure => 'present',
-  }
-  end
-
-  let :redhat_platform_params do {
-    :ovs_package_name      => 'openvswitch',
-    :ovs_service_name      => 'openvswitch',
-    :provider              => 'ovs_redhat',
-  }
-  end
-
-  let :debian_platform_params do {
-    :ovs_package_name      => 'openvswitch-switch',
-    :ovs_dkms_package_name => 'openvswitch-datapath-dkms',
-    :ovs_service_name      => 'openvswitch-switch',
-    :provider              => 'ovs',
-    :service_hasstatus     => false,
-    :service_status        => '/etc/init.d/openvswitch-switch status | fgrep -q "not running"; if [ $? -eq 0 ]; then exit 1; else exit 0; fi',
-  }
-  end
-
-  let :ubuntu_platform_params do {
-    :ovs_package_name      => 'openvswitch-switch',
-    :ovs_dkms_package_name => 'openvswitch-datapath-dkms',
-    :ovs_service_name      => 'openvswitch-switch',
-    :provider              => 'ovs',
-    :service_hasstatus     => false,
-    :service_status        => '/sbin/status openvswitch-switch | fgrep "start/running"',
+    :dkms_ensure => false,
+    :enable_hw_offload => false,
   }
   end
 
@@ -58,12 +33,22 @@ describe 'vswitch::ovs' do
   }
   end
 
-  shared_examples_for 'vswitch ovs' do
-    it 'contains params' do
-        is_expected.to contain_class('vswitch::params')
-    end
+  shared_examples_for 'vswitch::ovs' do
+    context 'default parameters' do
+      let (:params) { default_params }
 
-    it 'configures service' do
+      it 'contains the ovs class' do
+        is_expected.to contain_class('vswitch::ovs')
+      end
+
+      it 'contains params' do
+        is_expected.to contain_class('vswitch::params')
+      end
+
+      it 'configures hw-offload option to false' do
+          is_expected.to_not contain_vs_config('other_config:hw-offload')
+      end
+      it 'configures service' do
         is_expected.to contain_service('openvswitch').with(
           :ensure    => true,
           :enable    => true,
@@ -71,14 +56,66 @@ describe 'vswitch::ovs' do
           :hasstatus => platform_params[:service_hasstatus],
           :status    => platform_params[:service_status],
         )
+      end
+
+      it 'install package' do
+        is_expected.to contain_package(platform_params[:ovs_package_name]).with(
+          :name   => platform_params[:ovs_package_name],
+          :ensure => params[:package_ensure],
+          :before => 'Service[openvswitch]'
+        )
+      end
     end
 
-    it 'install package' do
-      is_expected.to contain_package(platform_params[:ovs_package_name]).with(
-        :name   => platform_params[:ovs_package_name],
-        :ensure => params[:package_ensure],
-        :before => 'Service[openvswitch]'
-      )
+    context 'custom parameters' do
+      let :params do
+        {
+          :package_ensure => 'latest',
+          :dkms_ensure    => false,
+          :enable_hw_offload => true,
+        }
+      end
+      it 'installs correct package' do
+        is_expected.to contain_package(platform_params[:ovs_package_name]).with(
+          :name   => platform_params[:ovs_package_name],
+          :ensure => 'latest',
+          :before => 'Service[openvswitch]'
+        )
+      end
+      it 'configures hw-offload option' do
+          is_expected.to contain_vs_config('other_config:hw-offload').with(
+            :value  => 'true', :notify => 'Service[openvswitch]', :wait => true,
+          )
+      end
+    end
+  end
+
+  shared_examples_for "vswitch::ovs on Debian" do
+    context 'with dkms ensure true' do
+      let (:params) do
+        {
+          :package_ensure => 'latest',
+          :dkms_ensure => true
+        }
+      end
+      it 'install kernel module' do
+        is_expected.to contain_package(platform_params[:ovs_dkms_package_name]).with(
+          :name   => platform_params[:ovs_dkms_package_name],
+          :ensure => params[:package_ensure],
+        )
+      end
+      it 'rebuilds kernel module' do
+        is_expected.to contain_exec('rebuild-ovsmod').with(
+          :command     => '/usr/sbin/dpkg-reconfigure openvswitch-datapath-dkms > /tmp/reconf-log',
+          :refreshonly => true,
+        )
+      end
+    end
+  end
+
+  shared_examples_for "vswitch::ovs on RedHat" do
+    it 'does not rebuild kernel module' do
+        is_expected.to_not contain_exec('rebuild-ovsmod')
     end
   end
 
@@ -103,115 +140,6 @@ describe 'vswitch::ovs' do
     end
   end
 
-  context 'on redhat with default parameters' do
-    let :params do default_params end
-
-    let :facts do
-      OSDefaults.get_facts({
-        :osfamily    => 'Redhat',
-        :ovs_version => '1.4.2',
-      })
-    end
-
-    let :platform_params do redhat_platform_params end
-
-    it_configures 'vswitch ovs'
-    it_configures 'do not install dkms'
-  end
-
-  context 'on redhat with parameters' do
-    let :params do {
-      :package_ensure => 'latest',
-      :dkms_ensure    => false,
-    }
-    end
-
-    let :facts do
-      OSDefaults.get_facts({
-        :osfamily    => 'Redhat',
-        :ovs_version => '1.4.2',
-      })
-    end
-    let :platform_params do redhat_platform_params end
-
-    it_configures 'vswitch ovs'
-    it_configures 'do not install dkms'
-  end
-
-  context 'on Debian with default parameters' do
-    let :params do default_params end
-
-    let :facts do
-      OSDefaults.get_facts({
-        :operatingsystem => 'Debian',
-        :osfamily        => 'Debian',
-        :ovs_version     => '1.4.2',
-      })
-    end
-    let :platform_params do debian_platform_params end
-
-    it_configures 'vswitch ovs'
-    it_configures 'do not install dkms'
-  end
-
-  context 'on Debian with parameters' do
-    let :params do {
-      :package_ensure => 'latest',
-      :dkms_ensure    => true,
-    }
-    end
-
-    let :facts do
-      OSDefaults.get_facts({
-        :operatingsystem => 'Debian',
-        :osfamily        => 'Debian',
-        :ovs_version     => '1.4.2',
-      })
-    end
-    let :platform_params do debian_platform_params end
-
-    it_configures 'vswitch ovs'
-    it_configures 'install dkms'
-  end
-
-  context 'on Ubuntu with default parameters' do
-    let :params do default_params end
-
-    let :facts do
-      OSDefaults.get_facts({
-        :operatingsystem           => 'Ubuntu',
-        :operatingsystemmajrelease => '14',
-        :osfamily                  => 'Debian',
-        :ovs_version               => '1.4.2',
-      })
-    end
-    let :platform_params do ubuntu_platform_params end
-
-    it_configures 'vswitch ovs'
-    it_configures 'do not install dkms'
-  end
-
-  context 'on Ubuntu with parameters' do
-    let :params do {
-      :package_ensure => 'latest',
-      :dkms_ensure    => true,
-    }
-    end
-
-    let :facts do
-      OSDefaults.get_facts({
-        :operatingsystem           => 'Ubuntu',
-        :operatingsystemmajrelease => '14',
-        :osfamily                  => 'Debian',
-        :ovs_version               => '1.4.2',
-      })
-    end
-    let :platform_params do ubuntu_platform_params end
-
-    it_configures 'vswitch ovs'
-    it_configures 'install dkms'
-  end
-
   context 'on FreeBSD with default parameters' do
     let :params do default_params end
 
@@ -224,7 +152,7 @@ describe 'vswitch::ovs' do
     end
     let :platform_params do freebsd_platform_params end
 
-    it_configures 'vswitch ovs'
+    it_configures 'vswitch::ovs'
     it_configures 'do not install dkms'
 
     it 'configures ovsdb service' do
@@ -253,7 +181,7 @@ describe 'vswitch::ovs' do
     end
     let :platform_params do freebsd_platform_params end
 
-    it_configures 'vswitch ovs'
+    it_configures 'vswitch::ovs'
     it_configures 'do not install dkms'
 
     it 'configures ovsdb service' do
@@ -283,7 +211,7 @@ describe 'vswitch::ovs' do
     end
     let :platform_params do solaris_platform_params end
 
-    it_configures 'vswitch ovs'
+    it_configures 'vswitch::ovs'
     it_configures 'do not install dkms'
 
     it 'configures ovsdb service' do
@@ -312,7 +240,7 @@ describe 'vswitch::ovs' do
     end
     let :platform_params do solaris_platform_params end
 
-    it_configures 'vswitch ovs'
+    it_configures 'vswitch::ovs'
     it_configures 'do not install dkms'
 
     it 'configures ovsdb service' do
@@ -326,4 +254,52 @@ describe 'vswitch::ovs' do
     end
   end
 
+  on_supported_os({
+    :supported_os => OSDefaults.get_supported_os
+  }).each do |os,facts|
+    context "on #{os}" do
+      let (:facts) do
+        facts.merge!(OSDefaults.get_facts({ :ovs_version => '1.4.2' }))
+      end
+
+      let (:platform_params) do
+        case facts[:osfamily]
+        when 'Debian'
+          if facts[:operatingsystem] == 'Debian'
+            {
+              :ovs_package_name      => 'openvswitch-switch',
+              :ovs_dkms_package_name => 'openvswitch-datapath-dkms',
+              :ovs_service_name      => 'openvswitch-switch',
+              :provider              => 'ovs',
+              :service_hasstatus     => true,
+            }
+          elsif facts[:operatingsystem] == 'Ubuntu'
+            if Puppet::Util::Package.versioncmp(facts[:operatingsystemmajrelease], '16') >= 0
+              status_cmd = '/etc/init.d/openvswitch-switch status | fgrep -q "not running"; if [ $? -eq 0 ]; then exit 1; else exit 0; fi'
+            else
+              status_cmd = '/sbin/status openvswitch-switch | fgrep "start/running"'
+            end
+            {
+              :ovs_package_name      => 'openvswitch-switch',
+              :ovs_dkms_package_name => 'openvswitch-datapath-dkms',
+              :ovs_service_name      => 'openvswitch-switch',
+              :provider              => 'ovs',
+              :service_hasstatus     => false,
+              :service_status        => status_cmd
+            }
+          end
+        when 'RedHat'
+          {
+            :ovs_package_name      => 'openvswitch',
+            :ovs_service_name      => 'openvswitch',
+            :provider              => 'ovs_redhat',
+          }
+        end
+      end
+
+      it_behaves_like "vswitch::ovs"
+      it_behaves_like "vswitch::ovs on #{facts[:osfamily]}"
+    end
+  end
+
 end
